@@ -10,10 +10,10 @@ TEST(client, tcp)
     ASSERT_EQ(ret, 0);
     ret = cli.connect(&cli, (char *) "127.0.0.1", 80, 0.5, 0);
     ASSERT_EQ(ret, 0);
-    ret = cli.send(&cli, (char *) SW_STRL("GET / HTTP/1.1\r\nHost: localhost\r\n\r\n"), 0);
+    ret = cli.send(&cli, (char *) SW_STRS("GET / HTTP/1.1\r\nHost: localhost\r\n\r\n"), 0);
     ASSERT_GT(ret, 0);
     ret = cli.recv(&cli, buf, 128, 0);
     ASSERT_GT(ret, 0);
     cli.close(&cli);
-    ASSERT_EQ(strncmp(buf, SW_STRL("HTTP/1.1 200 OK\r\n") -1), 0);
+    ASSERT_EQ(strncmp(buf, SW_STRL("HTTP/1.1 200 OK\r\n")), 0);
 }
