@@ -14,75 +14,103 @@
   +----------------------------------------------------------------------+
 */
 
-#ifndef SW_COROUTINE_H_
-#define SW_COROUTINE_H_
+#pragma once
 
 #include "swoole.h"
+#include "context.h"
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
+#include <string>
+#include <unordered_map>
 
 #define DEFAULT_MAX_CORO_NUM 3000
 #define DEFAULT_STACK_SIZE   8192
 #define MAX_CORO_NUM_LIMIT   0x80000
 
 #define CORO_END 0
-#define CORO_YIELD 1
 #define CORO_LIMIT -1
-#define CORO_SAVE 3
+#define CORO_INVALID -2
 
-typedef struct coroutine_s coroutine_t;
-typedef void (*coroutine_func_t)(void*);
+typedef enum
+{
+    SW_CORO_INIT = 0, SW_CORO_WAITING, SW_CORO_RUNNING, SW_CORO_END,
+} sw_coro_state;
 
 typedef void (*coro_php_create_t)();
 typedef void (*coro_php_yield_t)(void*);
 typedef void (*coro_php_resume_t)(void*);
 typedef void (*coro_php_close_t)();
 
-typedef enum
+namespace swoole
 {
-    SW_CORO_YIELD = 0, SW_CORO_SUSPENDED, SW_CORO_RUNNING, SW_CORO_END,
-} sw_coro_state;
+class Coroutine
+{
+private:
+    long cid;
+    void *task;
+    swoole::Context ctx;
 
+public:
+    sw_coro_state state;
 
-void coro_yield();
-void coro_handle_timeout();
+    Coroutine(long _cid, size_t stack_size, coroutine_func_t fn, void *private_data) :
+            ctx(stack_size, fn, private_data)
+    {
+        cid = _cid;
+        task = nullptr;
+        state = SW_CORO_INIT;
+    }
 
-/* basic api */
-int coroutine_create(coroutine_func_t func, void* args);
-void coroutine_resume(coroutine_t *co);
-void coroutine_yield(coroutine_t *co);
-void coroutine_resume_naked(coroutine_t *co);
-void coroutine_yield_naked(coroutine_t *co);
-void coroutine_release(coroutine_t *co);
+    void resume();
+    void yield();
+
+    void resume_naked();
+    void yield_naked();
+
+    void release();
+
+    inline void set_task(void *_task)
+    {
+        task = _task;
+    }
+
+    inline long get_cid()
+    {
+        return cid;
+    }
+
+    inline void* get_task()
+    {
+        return task;
+    }
+
+    static long create(coroutine_func_t fn, void* args);
+    static int sleep(double sec);
+    static swString* read_file(const char *file, int lock);
+    static ssize_t write_file(const char *file, char *buf, size_t length, int lock, int flags);
+};
+}
 /* co task */
-void coroutine_set_task(coroutine_t *co, void *ptr);
 void* coroutine_get_current_task();
-void* coroutine_get_task_by_cid(int cid);
+void* coroutine_get_task_by_cid(long cid);
 /* get coroutine */
-coroutine_t* coroutine_get_current();
-coroutine_t *coroutine_get_by_id(int cid);
+swoole::Coroutine* coroutine_get_current();
+swoole::Coroutine* coroutine_get_by_id(long cid);
 /* get cid */
-int coroutine_get_cid(coroutine_t *co);
-int coroutine_get_current_cid();
-/* cid api */
-int coroutine_test_alloc_cid();
-void coroutine_test_free_cid(int cid);
+long coroutine_get_current_cid();
 void coroutine_set_stack_size(int stack_size);
 /* callback */
 void coroutine_set_onYield(coro_php_yield_t func);
 void coroutine_set_onResume(coro_php_resume_t func);
+void coroutine_set_onResumeBack(coro_php_resume_t func);
 void coroutine_set_onClose(coro_php_close_t func);
 
-void internal_coro_yield(void *return_value);
-void internal_coro_resume(void *data);
+inline static long coroutine_get_cid(swoole::Coroutine* co)
+{
+    return co ? co->get_cid() : -1;
+}
 
-swString* swoole_coroutine_read_file(const char *file, int lock);
-ssize_t swoole_coroutine_write_file(const char *file, char *buf, size_t length, int lock, int flags);
+void internal_coro_yield(void *arg);
+void internal_coro_resume(void *arg);
+
+std::unordered_map<long, swoole::Coroutine*>* coroutine_get_map();
 
-#ifdef __cplusplus
-}  /* end extern "C" */
-#endif
-#endif
