@@ -35,6 +35,7 @@ enum swErrorCode
     SW_ERROR_DNSLOOKUP_DUPLICATE_REQUEST,
     SW_ERROR_DNSLOOKUP_RESOLVE_FAILED,
     SW_ERROR_BAD_IPV6_ADDRESS,
+    SW_ERROR_UNREGISTERED_SIGNAL,
 
     /**
      * connection error
@@ -104,9 +105,9 @@ enum swErrorCode
     SW_ERROR_SERVER_TOO_MANY_LISTEN_PORT,
     SW_ERROR_SERVER_PIPE_BUFFER_FULL,
     SW_ERROR_SERVER_NO_IDLE_WORKER,
-
     SW_ERROR_SERVER_ONLY_START_ONE,
     SW_ERROR_SERVER_SEND_IN_MASTER,
+    SW_ERROR_SERVER_INVALID_REQUEST,
 
     /**
      * Process exit timeout, forced to end.
