@@ -437,13 +437,13 @@ struct _swServer
      */
     uint32_t daemonize :1;
     /**
-     * have udp listen socket
+     * have dgram socket
      */
-    uint32_t have_udp_sock :1;
+    uint32_t have_dgram_sock :1;
     /**
-     * have tcp listen socket
+     * have stream socket
      */
-    uint32_t have_tcp_sock :1;
+    uint32_t have_stream_sock :1;
     /**
      * open cpu affinity setting
      */
@@ -1071,7 +1071,8 @@ int swReactorProcess_onClose(swReactor *reactor, swEvent *event);
 
 int swManager_start(swFactory *factory);
 pid_t swManager_spawn_user_worker(swServer *serv, swWorker* worker);
-int swManager_wait_user_worker(swProcessPool *pool, pid_t pid, int status);
+pid_t swManager_spawn_task_worker(swServer *serv, swWorker* worker);
+int swManager_wait_other_worker(swProcessPool *pool, pid_t pid, int status);
 void swManager_kill_user_worker(swServer *serv);
 
 #ifdef __cplusplus
