@@ -87,7 +87,7 @@ int daemon(int nochdir, int noclose);
 
 /*----------------------------------------------------------------------------*/
 
-#define SWOOLE_VERSION "4.2.7"
+#define SWOOLE_VERSION "4.2.9"
 #define SWOOLE_BUG_REPORT \
     "A bug occurred in Swoole-v" SWOOLE_VERSION ", please report it.\n"\
     "The Swoole developers probably don't know about it,\n"\
@@ -413,6 +413,7 @@ enum swTraceType
     SW_TRACE_AIO              = 1u << 18,
     SW_TRACE_SSL              = 1u << 19,
     SW_TRACE_NORMAL           = 1u << 20,
+    SW_TRACE_CHANNEL          = 1u << 21,
 };
 
 #ifdef SW_LOG_TRACE_OPEN
@@ -1757,7 +1758,7 @@ int swReactor_setHandle(swReactor *, int, swReactor_handle);
 int swReactor_empty(swReactor *reactor);
 
 void swReactor_defer_task_create(swReactor *reactor);
-void swReactor_defer_task_destory(swReactor *reactor);
+void swReactor_defer_task_destroy(swReactor *reactor);
 
 static sw_inline swConnection* swReactor_get(swReactor *reactor, int fd)
 {
