@@ -39,8 +39,7 @@ static sw_inline int add_assoc_ulong_safe(zval *arg, const char *key, zend_ulong
     }
     else
     {
-        char buf[MAX_LENGTH_OF_LONG + 1];
-        memset((char *) buf, 0, MAX_LENGTH_OF_LONG + 1);
+        char buf[MAX_LENGTH_OF_LONG + 1] = {0};
         sprintf((char *) buf, ZEND_ULONG_FMT, value);
         return add_assoc_string(arg, key, buf);
     }
@@ -80,6 +79,23 @@ static sw_inline int sw_call_user_function_ex(HashTable *function_table, zval* o
     return call_user_function_ex(function_table, object_p, function_name, &_retval, param_count, param_count ? params : NULL, no_separation, ymbol_table);
 }
 
+static sw_inline int sw_call_user_function_anyway(zval *object_p, zval *function_name, zval **retval_ptr_ptr, uint32_t param_count, zval params[], int no_separation)
+{
+    static zval _retval;
+    *retval_ptr_ptr = &_retval;
+    zend_object* exception = EG(exception);
+    if (exception)
+    {
+        EG(exception) = NULL;
+    }
+    int ret = call_user_function_ex(NULL, object_p, function_name, &_retval, param_count, param_count ? params : NULL, no_separation, NULL);
+    if (exception)
+    {
+        EG(exception) = exception;
+    }
+    return ret;
+}
+
 static sw_inline int sw_call_user_function_fast_ex(zval *function_name, zend_fcall_info_cache *fci_cache, zval **retval_ptr_ptr, uint32_t param_count, zval *params)
 {
     static zval _retval;
@@ -92,7 +108,14 @@ static sw_inline int sw_call_user_function_fast_ex(zval *function_name, zend_fca
     fci.symbol_table = NULL;
 #endif
     fci.object = NULL;
-    ZVAL_COPY_VALUE(&fci.function_name, function_name);
+    if (!fci_cache || !fci_cache->function_handler)
+    {
+        ZVAL_COPY_VALUE(&fci.function_name, function_name);
+    }
+    else
+    {
+        ZVAL_UNDEF(&fci.function_name);
+    }
     fci.retval = *retval_ptr_ptr;
     fci.param_count = param_count;
     fci.params = params;
@@ -192,10 +215,10 @@ static sw_inline int sw_zend_is_callable(zval *cb, int a, char **name)
     return ret;
 }
 
-static inline int sw_zend_is_callable_ex(zval *zcallable, zval *zobject, uint check_flags, char **callable_name, int *callable_name_len, zend_fcall_info_cache *fcc, char **error)
+static sw_inline int sw_zend_is_callable_ex(zval *zcallable, zval *zobject, uint check_flags, char **callable_name, int *callable_name_len, zend_fcall_info_cache *fci_cache, char **error)
 {
     zend_string *key = NULL;
-    int ret = zend_is_callable_ex(zcallable, NULL, check_flags, &key, fcc, error);
+    int ret = zend_is_callable_ex(zcallable, NULL, check_flags, &key, fci_cache, error);
     char *tmp = estrndup(key->val, key->len);
     zend_string_release(key);
     *callable_name = tmp;
