@@ -33,6 +33,7 @@
 #include "zend_interfaces.h"
 #include "zend_exceptions.h"
 #include "zend_variables.h"
+#include "zend_closures.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -74,11 +75,6 @@ BEGIN_EXTERN_C()
 #define SW_HOST_SIZE  128
 
 extern PHPAPI int php_array_merge(HashTable *dest, HashTable *src);
-typedef struct
-{
-    uint16_t port;
-    uint16_t from_fd;
-} php_swoole_udp_t;
 
 extern zend_module_entry swoole_module_entry;
 
@@ -248,6 +244,12 @@ typedef enum
     PHP_SWOOLE_RSHUTDOWN_END,
 } php_swoole_req_status;
 //---------------------------------------------------------
+typedef struct
+{
+    zval _callback;
+    zval *callback;
+} php_defer_callback;
+//---------------------------------------------------------
 #define php_swoole_socktype(type)           (type & (~SW_FLAG_SYNC) & (~SW_FLAG_ASYNC) & (~SW_FLAG_KEEP) & (~SW_SOCK_SSL))
 #define php_swoole_array_length(array)      zend_hash_num_elements(Z_ARRVAL_P(array))
 
@@ -285,6 +287,7 @@ PHP_FUNCTION(swoole_call_user_shutdown_begin);
 PHP_FUNCTION(swoole_coroutine_create);
 PHP_FUNCTION(swoole_coroutine_exec);
 PHP_FUNCTION(swoole_coroutine_gethostbyname);
+PHP_FUNCTION(swoole_coroutine_defer);
 //---------------------------------------------------------
 //                  swoole_server
 //---------------------------------------------------------
@@ -386,7 +389,7 @@ PHP_FUNCTION(swoole_fast_serialize);
 PHP_FUNCTION(swoole_unserialize);
 #endif
 
-void swoole_destory_table(zend_resource *rsrc);
+void swoole_destroy_table(zend_resource *rsrc);
 
 void swoole_server_port_init(int module_number);
 void swoole_async_init(int module_number);
@@ -404,6 +407,7 @@ void swoole_postgresql_coro_init (int module_number);
 void swoole_mysql_coro_init(int module_number);
 void swoole_http_client_coro_init(int module_number);
 void swoole_coroutine_util_init(int module_number);
+void swoole_coroutine_util_destroy();
 void swoole_http_client_init(int module_number);
 void swoole_redis_init(int module_number);
 void swoole_redis_server_init(int module_number);
@@ -426,6 +430,7 @@ void swoole_serialize_init(int module_number);
 #endif
 void swoole_memory_pool_init(int module_number);
 
+void php_swoole_process_clean();
 int php_swoole_process_start(swWorker *process, zval *zobject);
 
 void php_swoole_reactor_init();
@@ -456,7 +461,6 @@ void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset);
 void php_swoole_websocket_frame_unpack(swString *data, zval *zframe);
 int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opcode, zend_bool fin, zend_bool mask);
 void php_swoole_sha1(const char *str, int _len, unsigned char *digest);
-int php_swoole_client_isset_callback(zval *zobject, int type);
 
 int php_swoole_task_pack(swEventData *task, zval *data);
 zval* php_swoole_task_unpack(swEventData *task_result);
@@ -490,7 +494,7 @@ php_socket *swoole_convert_to_socket(int sock);
 #endif
 
 zval* php_swoole_server_get_callback(swServer *serv, int server_fd, int event_type);
-zend_fcall_info_cache* php_swoole_server_get_cache(swServer *serv, int server_fd, int event_type);
+zend_fcall_info_cache* php_swoole_server_get_fci_cache(swServer *serv, int server_fd, int event_type);
 void php_swoole_server_before_start(swServer *serv, zval *zobject);
 void php_swoole_http_server_before_start(swServer *serv, zval *zobject);
 void php_swoole_server_send_yield(swServer *serv, int fd, zval *zdata, zval *return_value);
@@ -510,6 +514,7 @@ int php_swoole_client_onPackage(swConnection *conn, char *data, uint32_t length)
 void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode);
 void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode);
 zend_bool php_swoole_signal_isset_handler(int signo);
+void php_swoole_event_onDefer(void *_cb);
 
 #ifdef SW_USE_FAST_SERIALIZE
 PHPAPI zend_string* php_swoole_serialize(zval *zvalue);
@@ -518,7 +523,7 @@ PHPAPI int php_swoole_unserialize(void *buffer, size_t len, zval *return_value,
 
 #ifdef SW_COROUTINE
 int php_coroutine_reactor_can_exit(swReactor *reactor);
-void sw_coro_check_bind(const char *name, int bind_cid);
+void sw_coro_check_bind(const char *name, long bind_cid);
 #endif
 
 #ifdef SW_USE_OPENSSL
@@ -534,7 +539,7 @@ static sw_inline int php_swoole_is_callable(zval *callback)
     char *func_name = NULL;
     if (!sw_zend_is_callable(callback, 0, &func_name))
     {
-        swoole_php_fatal_error(E_WARNING, "Function '%s' is not callable", func_name);
+        swoole_php_fatal_error(E_WARNING, "function '%s' is not callable", func_name);
         efree(func_name);
         return SW_FALSE;
     }
@@ -642,6 +647,17 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
             _real_arg = ZEND_CALL_ARG(execute_data, 0);
 #endif
 
+/* PHP 7.0 compatibility macro {{{*/
+#if PHP_VERSION_ID < 70100
+#define SW_DECLARE_EG_SCOPE(_scope) zend_class_entry *_scope
+#define SW_SAVE_EG_SCOPE(_scope) _scope = EG(scope)
+#define SW_SET_EG_SCOPE(_scope) EG(scope) = _scope
+#else
+#define SW_DECLARE_EG_SCOPE(_scope)
+#define SW_SAVE_EG_SCOPE(scope)
+#define SW_SET_EG_SCOPE(scope)
+#endif/*}}}*/
+
 /* PHP 7.3 forward compatibility */
 #ifndef GC_SET_REFCOUNT
 # define GC_SET_REFCOUNT(p, rc) do { \
@@ -658,6 +674,15 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
 	ZEND_HASH_DEC_APPLY_COUNT(p)
 #endif
 
+/* PHP 7.3 compatibility macro {{{*/
+#ifndef ZEND_CLOSURE_OBJECT
+# define ZEND_CLOSURE_OBJECT(func) (zend_object*)func->op_array.prototype
+#endif
+#ifndef GC_ADDREF
+# define GC_ADDREF(ref) ++GC_REFCOUNT(ref)
+# define GC_DELREF(ref) --GC_REFCOUNT(ref)
+#endif/*}}}*/
+
 #ifndef ZEND_HASH_APPLY_PROTECTION
 # define ZEND_HASH_APPLY_PROTECTION(p) 1
 #endif
