@@ -19,76 +19,40 @@
 #include <unordered_map>
 
 using namespace swoole;
-static std::unordered_map<void *, bool> channels;
 
-static void channel_defer_callback(void *data)
-{
-    notify_msg_t *msg = (notify_msg_t*) data;
-    if (channels.find((void*) msg->chan) != channels.end())
-    {
-        coroutine_t *co = msg->chan->pop_coroutine(msg->type);
-        if (co == nullptr)
-        {
-            return;
-        }
-        coroutine_resume(co);
-    }
-    delete msg;
-}
-
-static void channel_pop_timeout(swTimer *timer, swTimer_node *tnode)
+static void channel_operation_timeout(swTimer *timer, swTimer_node *tnode)
 {
     timeout_msg_t *msg = (timeout_msg_t *) tnode->data;
     msg->error = true;
     msg->timer = nullptr;
     msg->chan->remove(msg->co);
-    coroutine_resume(msg->co);
+    msg->co->resume();
 }
 
 Channel::Channel(size_t _capacity)
 {
     capacity = _capacity;
     closed = false;
-    notify_producer_count = 0;
-    notify_consumer_count = 0;
-    channels[(void*) this] = true;
 }
 
 void Channel::yield(enum channel_op type)
 {
-    int _cid = coroutine_get_current_cid();
-    if (_cid == -1)
+    Coroutine *co = coroutine_get_current();
+    if (unlikely(!co))
     {
         swError("Channel::yield() must be called in the coroutine.");
     }
-    coroutine_t *co = coroutine_get_by_id(_cid);
     if (type == PRODUCER)
     {
         producer_queue.push_back(co);
-        swDebug("producer[%d]", coroutine_get_cid(co));
+        swTraceLog(SW_TRACE_CHANNEL, "producer cid=%ld", co->get_cid());
     }
     else
     {
         consumer_queue.push_back(co);
-        swDebug("consumer[%d]", coroutine_get_cid(co));
+        swTraceLog(SW_TRACE_CHANNEL, "consumer cid=%ld", co->get_cid());
     }
-    coroutine_yield(co);
-}
-
-void Channel::notify(enum channel_op type)
-{
-    notify_msg_t *msg = new notify_msg_t;
-    msg->chan = this;
-    msg->type = type;
-    if (type == PRODUCER)
-    {
-        notify_producer_count++;
-    }
-    else
-    {
-        notify_consumer_count++;
-    }
-    SwooleG.main_reactor->defer(SwooleG.main_reactor, channel_defer_callback, msg);
+    co->yield();
 }
 
 void* Channel::pop(double timeout)
@@ -97,30 +61,29 @@ void* Channel::pop(double timeout)
     {
         return nullptr;
     }
-    timeout_msg_t msg;
-    msg.error = false;
-    if (timeout > 0)
-    {
-        int msec = (int) (timeout * 1000);
-        msg.chan = this;
-        msg.co = coroutine_get_by_id(coroutine_get_current_cid());
-        msg.timer = swTimer_add(&SwooleG.timer, msec, 0, &msg, channel_pop_timeout);
-    }
-    else
-    {
-        msg.timer = NULL;
-    }
     if (is_empty() || consumer_queue.size() > 0)
     {
+        timeout_msg_t msg;
+        msg.error = false;
+        msg.timer = NULL;
+        if (timeout > 0)
+        {
+            int msec = (int) (timeout * 1000);
+            msg.chan = this;
+            msg.co = coroutine_get_current();
+            msg.timer = swTimer_add(&SwooleG.timer, msec, 0, &msg, channel_operation_timeout);
+        }
+
         yield(CONSUMER);
-    }
-    if (msg.timer)
-    {
-        swTimer_del(&SwooleG.timer, msg.timer);
-    }
-    if (msg.error || closed || data_queue.size() == 0)
-    {
-        return nullptr;
+
+        if (msg.timer)
+        {
+            swTimer_del(&SwooleG.timer, msg.timer);
+        }
+        if (msg.error || closed)
+        {
+            return nullptr;
+        }
     }
     /**
      * pop data
@@ -130,14 +93,15 @@ void* Channel::pop(double timeout)
     /**
      * notify producer
      */
-    if (producer_queue.size() > 0 && notify_producer_count < producer_queue.size())
+    if (producer_queue.size() > 0)
     {
-        notify(PRODUCER);
+        Coroutine *co = pop_coroutine(PRODUCER);
+        co->resume();
     }
     return data;
 }
 
-bool Channel::push(void *data)
+bool Channel::push(void *data, double timeout)
 {
     if (closed)
     {
@@ -145,23 +109,40 @@ bool Channel::push(void *data)
     }
     if (is_full() || producer_queue.size() > 0)
     {
+        timeout_msg_t msg;
+        msg.error = false;
+        msg.timer = NULL;
+        if (timeout > 0)
+        {
+            int msec = (int) (timeout * 1000);
+            msg.chan = this;
+            msg.co = coroutine_get_current();
+            msg.timer = swTimer_add(&SwooleG.timer, msec, 0, &msg, channel_operation_timeout);
+        }
+
         yield(PRODUCER);
-    }
-    if (closed)
-    {
-        return false;
+
+        if (msg.timer)
+        {
+            swTimer_del(&SwooleG.timer, msg.timer);
+        }
+        if (msg.error || closed)
+        {
+            return false;
+        }
     }
     /**
      * push data
      */
     data_queue.push(data);
-    swDebug("push data, count=%ld", length());
+    swTraceLog(SW_TRACE_CHANNEL, "push data to channel, count=%ld", length());
     /**
      * notify consumer
      */
-    if (consumer_queue.size() > 0 && notify_consumer_count < consumer_queue.size())
+    if (consumer_queue.size() > 0)
     {
-        notify(CONSUMER);
+        Coroutine *co = pop_coroutine(CONSUMER);
+        co->resume();
     }
     return true;
 }
@@ -172,20 +153,17 @@ bool Channel::close()
     {
         return false;
     }
-    swDebug("closed");
+    swTraceLog(SW_TRACE_CHANNEL, "channel closed");
     closed = true;
-    while (producer_queue.size() > 0 && notify_producer_count < producer_queue.size())
+    while (producer_queue.size() > 0)
     {
-        notify(PRODUCER);
+        Coroutine *co = pop_coroutine(PRODUCER);
+        co->resume();
     }
-    while (consumer_queue.size() > 0 && notify_consumer_count < consumer_queue.size())
+    while (consumer_queue.size() > 0)
     {
-        notify(CONSUMER);
+        Coroutine *co = pop_coroutine(CONSUMER);
+        co->resume();
     }
     return true;
 }
-
-Channel::~Channel()
-{
-    channels.erase((void*) this);
-}
