@@ -1,5 +1,5 @@
 #include "socket.h"
-#include "context.h"
+#include "coroutine.h"
 #include "async.h"
 #include "buffer.h"
 
@@ -657,13 +657,17 @@ ssize_t Socket::recv(void *__buf, size_t __n)
         if (socket->ssl && socket->ssl_want_write)
         {
             events = SW_EVENT_WRITE;
+            if (!is_available(write_cid))
+            {
+                return -1;
+            }
         }
 #endif
         if (!wait_events(events))
         {
             return -1;
         }
-        yield(SOCKET_LOCK_READ);
+        yield(events == SW_EVENT_WRITE ? SOCKET_LOCK_RW : SOCKET_LOCK_READ);
         if (errCode == ETIMEDOUT)
         {
             return -1;
@@ -707,13 +711,17 @@ ssize_t Socket::read(void *__buf, size_t __n)
         if (socket->ssl && socket->ssl_want_write)
         {
             events = SW_EVENT_WRITE;
+            if (!is_available(write_cid))
+            {
+                return -1;
+            }
         }
 #endif
         if (!wait_events(events))
         {
             return -1;
         }
-        yield(SOCKET_LOCK_READ);
+        yield(events == SW_EVENT_WRITE ? SOCKET_LOCK_RW : SOCKET_LOCK_READ);
         if (errCode == ETIMEDOUT)
         {
             return -1;
@@ -757,18 +765,22 @@ ssize_t Socket::write(const void *__buf, size_t __n)
         if (socket->ssl && socket->ssl_want_read)
         {
             events = SW_EVENT_READ;
+            if (!is_available(read_cid))
+            {
+                return -1;
+            }
         }
 #endif
         if (!wait_events(events))
         {
             return -1;
         }
-        yield(SOCKET_LOCK_WRITE);
+        yield(events == SW_EVENT_READ ? SOCKET_LOCK_RW : SOCKET_LOCK_WRITE);
         if (errCode == ETIMEDOUT)
         {
             return -1;
         }
-        ssize_t retval =  ::write(socket->fd, (void *) __buf, __n);
+        retval =  ::write(socket->fd, (void *) __buf, __n);
         if (retval < 0)
         {
             if (swConnection_error(errno) == SW_WAIT)
@@ -820,6 +832,7 @@ ssize_t Socket::send_all(const void *__buf, size_t __n)
     while (true)
     {
         retval = send((char*) __buf + total_bytes, __n - total_bytes);
+        swTrace("send retval=%ld", retval);
         if (retval <= 0)
         {
             if (total_bytes == 0)
@@ -862,18 +875,22 @@ ssize_t Socket::send(const void *__buf, size_t __n)
         if (socket->ssl && socket->ssl_want_read)
         {
             events = SW_EVENT_READ;
+            if (!is_available(read_cid))
+            {
+                return -1;
+            }
         }
 #endif
         if (!wait_events(events))
         {
             return -1;
         }
-        yield(SOCKET_LOCK_WRITE);
+        yield(events == SW_EVENT_READ ? SOCKET_LOCK_RW : SOCKET_LOCK_WRITE);
         if (errCode == ETIMEDOUT)
         {
             return -1;
         }
-        ssize_t retval = swConnection_send(socket, (void *) __buf, __n, 0);
+        retval = swConnection_send(socket, (void *) __buf, __n, 0);
         if (retval < 0)
         {
             if (swConnection_error(errno) == SW_WAIT)
@@ -926,9 +943,8 @@ ssize_t Socket::sendmsg(const struct msghdr *msg, int flags)
 
 ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
 {
-    if (read_cid)
+    if (!is_available(read_cid))
     {
-        swWarn("socket has already been bound to another coroutine #%d.", read_cid);
         return -1;
     }
     ssize_t retval = ::recvmsg(socket->fd, msg, flags);
@@ -963,48 +979,43 @@ ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
 
 void Socket::yield(int operation)
 {
-    int cid = coroutine_get_current_cid();
-    if (unlikely(cid == -1))
+    Coroutine *co = coroutine_get_current();
+    if (unlikely(!co))
     {
         swError("Socket::yield() must be called in the coroutine.");
     }
 
+    int cid = co->get_cid();
     errCode = 0;
+    int ms = (int) (_timeout * 1000);
+    if (ms <= 0 || ms >= SW_TIMER_MAX_VALUE)
+    {
+        _timeout = -1;
+    }
     if (_timeout > 0)
     {
-        int ms = (int) (_timeout * 1000);
-        if (ms <= 0 || ms >= SW_TIMER_MAX_VALUE)
+        swTimer_node *timer = swTimer_add(&SwooleG.timer, ms, 0, this, socket_onTimeout);
+        if (timer)
         {
-            goto _skip_timer;
-        }
-        if (operation == SOCKET_LOCK_WRITE)
-        {
-            write_timer = swTimer_add(&SwooleG.timer, ms, 0, this, socket_onTimeout);
-            if (write_timer)
+            if (operation == SOCKET_LOCK_WRITE)
             {
+                write_timer = timer;
                 write_timer->type = SW_TIMER_TYPE_CORO_WRITE;
             }
-        }
-        else if (operation == SOCKET_LOCK_READ)
-        {
-            read_timer = swTimer_add(&SwooleG.timer, ms, 0, this, socket_onTimeout);
-            if (read_timer)
+            else if (operation == SOCKET_LOCK_READ)
             {
+                read_timer = timer;
                 read_timer->type = SW_TIMER_TYPE_CORO_READ;
             }
-        }
-        else
-        {
-            read_timer = swTimer_add(&SwooleG.timer, ms, 0, this, socket_onTimeout);
-            if (read_timer == nullptr)
+            else // if (operation == SOCKET_LOCK_RW)
             {
+                read_timer = timer;
                 read_timer->type = SW_TIMER_TYPE_CORO_ALL;
             }
         }
     }
 
     // bind read/write coroutine
-    _skip_timer:
     if (operation & SOCKET_LOCK_WRITE)
     {
         write_cid = cid;
@@ -1014,9 +1025,7 @@ void Socket::yield(int operation)
         read_cid = cid;
     }
     //=== yield ===
-
-    coroutine_yield(coroutine_get_by_id(cid));
-
+    co->yield();
     //=== resume ===
     if (operation & SOCKET_LOCK_WRITE)
     {
@@ -1042,7 +1051,7 @@ void Socket::yield(int operation)
 
 void Socket::resume(int operation)
 {
-    int cid = 0;
+    long cid = 0;
     if (operation & SOCKET_LOCK_READ)
     {
         cid = read_cid;
@@ -1055,7 +1064,7 @@ void Socket::resume(int operation)
     {
         assert(0);
     }
-    coroutine_resume(coroutine_get_by_id(cid));
+    coroutine_get_by_id(cid)->resume();
 }
 
 bool Socket::bind(std::string address, int port)
@@ -1197,14 +1206,15 @@ Socket* Socket::accept()
     }
     if (conn < 0)
     {
-        _error:
         errCode = errno;
         return nullptr;
     }
     Socket *client_sock = new Socket(conn, this);
-    if (client_sock->get_fd() < 0)
+    if (unlikely(client_sock->socket == nullptr))
     {
-        goto _error;
+        errCode = errno;
+        delete client_sock;
+        return nullptr;
     }
     memcpy(&client_sock->socket->info.addr, &client_addr.addr, client_addr.len);
 #ifdef SW_USE_OPENSSL
@@ -1537,7 +1547,7 @@ bool Socket::sendfile(char *filename, off_t offset, size_t length)
         {
             goto _error;
         }
-        yield(SOCKET_LOCK_READ);
+        yield(SOCKET_LOCK_WRITE);
         if (errCode == ETIMEDOUT)
         {
             goto _error;
