@@ -45,7 +45,7 @@ static sw_inline void swServer_reactor_schedule(swServer *serv)
 #endif
 
 static int swServer_start_check(swServer *serv);
-static void swServer_signal_hanlder(int sig);
+static void swServer_signal_handler(int sig);
 static int swServer_start_proxy(swServer *serv);
 static void swServer_disable_accept(swReactor *reactor);
 
@@ -211,20 +211,17 @@ int swServer_master_onAccept(swReactor *reactor, swEvent *event)
 
 static int swServer_start_check(swServer *serv)
 {
-    if (serv->onReceive == NULL && serv->onPacket == NULL)
-    {
-        swWarn("onReceive and onPacket event callback must be set.");
-        return SW_ERR;
-    }
-    if (serv->have_tcp_sock && serv->onReceive == NULL)
+    //stream
+    if (serv->have_stream_sock && serv->onReceive == NULL)
     {
         swWarn("onReceive event callback must be set.");
         return SW_ERR;
     }
-    //UDP
-    if (!serv->onPacket)
+    //dgram
+    if (serv->have_dgram_sock && serv->onPacket == NULL)
     {
-        serv->onPacket = serv->onReceive;
+        swWarn("onPacket event callback must be set.");
+        return SW_ERR;
     }
     //disable notice when use SW_DISPATCH_ROUND and SW_DISPATCH_QUEUE
     if (serv->factory_mode == SW_MODE_PROCESS)
@@ -247,11 +244,6 @@ static int swServer_start_check(swServer *serv)
             swWarn("onTask is null");
             return SW_ERR;
         }
-        if (serv->onFinish == NULL)
-        {
-            swWarn("onFinish is null");
-            return SW_ERR;
-        }
         if (serv->task_worker_num > SW_CPU_NUM * SW_MAX_WORKER_NCPU)
         {
             swWarn("serv->task_worker_num > %d, Too many processes, the system will be slow", SW_CPU_NUM * SW_MAX_WORKER_NCPU);
@@ -272,21 +264,24 @@ static int swServer_start_check(swServer *serv)
     {
         serv->reactor_num = serv->worker_num;
     }
-    if (SwooleG.max_sockets > 0 && serv->max_connection > SwooleG.max_sockets)
+    // max connections
+    uint32_t minimum_connection = (serv->worker_num + serv->task_worker_num) * 2 + 32;
+    if (serv->max_connection < minimum_connection)
     {
-        swWarn("serv->max_connection is exceed the maximum value[%d].", SwooleG.max_sockets);
         serv->max_connection = SwooleG.max_sockets;
+        swWarn("serv->max_connection must be bigger than %u, it's reset to %u", minimum_connection, SwooleG.max_sockets);
     }
-    if (serv->max_connection < (serv->worker_num + serv->task_worker_num) * 2 + 32)
+    else if (SwooleG.max_sockets > 0 && serv->max_connection > SwooleG.max_sockets)
     {
-        swWarn("serv->max_connection is too small.");
         serv->max_connection = SwooleG.max_sockets;
+        swWarn("serv->max_connection is exceed the maximum value, it's reset to %u.", SwooleG.max_sockets);
     }
-    if (serv->max_connection > SW_SESSION_LIST_SIZE)
+    else if (serv->max_connection > SW_SESSION_LIST_SIZE)
     {
-        swWarn("serv->max_connection is exceed the SW_SESSION_LIST_SIZE[%d].", SW_SESSION_LIST_SIZE);
         serv->max_connection = SW_SESSION_LIST_SIZE;
+        swWarn("serv->max_connection is exceed the SW_SESSION_LIST_SIZE, it's reset to %u.", SW_SESSION_LIST_SIZE);
     }
+    // package max length
     swListenPort *ls;
     LL_FOREACH(serv->listen_list, ls)
     {
@@ -805,7 +800,7 @@ void swServer_init(swServer *serv)
     serv->dispatch_mode = SW_DISPATCH_FDMOD;
 
     serv->worker_num = SW_CPU_NUM;
-    serv->max_connection = SwooleG.max_sockets < SW_SESSION_LIST_SIZE ? SwooleG.max_sockets : SW_SESSION_LIST_SIZE;
+    serv->max_connection = MIN(SW_MAX_CONNECTION, SwooleG.max_sockets);
 
     serv->max_request = 0;
     serv->max_wait_time = SW_WORKER_MAX_WAIT_TIME;
@@ -1203,17 +1198,17 @@ void swServer_signal_init(swServer *serv)
     swSignal_add(SIGHUP, NULL);
     if (serv->factory_mode == SW_MODE_PROCESS)
     {
-        swSignal_add(SIGCHLD, swServer_signal_hanlder);
+        swSignal_add(SIGCHLD, swServer_signal_handler);
     }
-    swSignal_add(SIGUSR1, swServer_signal_hanlder);
-    swSignal_add(SIGUSR2, swServer_signal_hanlder);
-    swSignal_add(SIGTERM, swServer_signal_hanlder);
+    swSignal_add(SIGUSR1, swServer_signal_handler);
+    swSignal_add(SIGUSR2, swServer_signal_handler);
+    swSignal_add(SIGTERM, swServer_signal_handler);
 #ifdef SIGRTMIN
-    swSignal_add(SIGRTMIN, swServer_signal_hanlder);
+    swSignal_add(SIGRTMIN, swServer_signal_handler);
 #endif
     swSignal_add(SIGALRM, swSystemTimer_signal_handler);
     //for test
-    swSignal_add(SIGVTALRM, swServer_signal_hanlder);
+    swSignal_add(SIGVTALRM, swServer_signal_handler);
     swServer_set_minfd(SwooleG.serv, SwooleG.signal_fd);
 }
 
@@ -1423,7 +1418,7 @@ int swserver_add_systemd_socket(swServer *serv)
 
         if (swSocket_is_dgram(ls->type))
         {
-            serv->have_udp_sock = 1;
+            serv->have_dgram_sock = 1;
             serv->dgram_port_num++;
             if (ls->type == SW_SOCK_UDP)
             {
@@ -1436,7 +1431,7 @@ int swserver_add_systemd_socket(swServer *serv)
         }
         else
         {
-            serv->have_tcp_sock = 1;
+            serv->have_stream_sock = 1;
         }
 
         LL_APPEND(serv->listen_list, ls);
@@ -1519,7 +1514,7 @@ swListenPort* swServer_add_port(swServer *serv, int type, char *host, int port)
 
     if (swSocket_is_dgram(ls->type))
     {
-        serv->have_udp_sock = 1;
+        serv->have_dgram_sock = 1;
         serv->dgram_port_num++;
         if (ls->type == SW_SOCK_UDP)
         {
@@ -1532,7 +1527,7 @@ swListenPort* swServer_add_port(swServer *serv, int type, char *host, int port)
     }
     else
     {
-        serv->have_tcp_sock = 1;
+        serv->have_stream_sock = 1;
     }
 
     LL_APPEND(serv->listen_list, ls);
@@ -1562,7 +1557,7 @@ int swServer_get_socket(swServer *serv, int port)
     return SW_ERR;
 }
 
-static void swServer_signal_hanlder(int sig)
+static void swServer_signal_handler(int sig)
 {
     swTraceLog(SW_TRACE_SERVER, "signal[%d] triggered.", sig);
 
