@@ -187,7 +187,8 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
 
     std::string key(Z_STRVAL_P(req->domain), Z_STRLEN_P(req->domain));
     dns_cache *cache;
-    if (request_cache_map.find(key) == request_cache_map.end())
+    auto cache_iterator = request_cache_map.find(key);
+    if (cache_iterator == request_cache_map.end())
     {
         cache = (dns_cache *) emalloc(sizeof(dns_cache));
         request_cache_map[key] = cache;
@@ -195,7 +196,7 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
     }
     else
     {
-        cache = request_cache_map[key];
+        cache = cache_iterator->second;
     }
 
     swString_write_ptr(cache->zaddress, 0, Z_STRVAL_P(zaddress), Z_STRLEN_P(zaddress));
@@ -214,7 +215,7 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
         return;
     }
 
-    int ret = coro_resume(req->context, zaddress, &retval);
+    int ret = sw_coro_resume(req->context, zaddress, retval);
     if (ret > 0)
     {
         goto free_zdata;
@@ -249,7 +250,7 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode)
         ZVAL_STRING(zaddress, "");
     }
 
-    int ret = coro_resume(req->context, zaddress, &retval);
+    int ret = sw_coro_resume(req->context, zaddress, retval);
     if (ret > 0)
     {
         goto free_zdata;
@@ -595,7 +596,8 @@ PHP_FUNCTION(swoole_async_write)
 
     int fd;
     std::string key(Z_STRVAL_P(filename), Z_STRLEN_P(filename));
-    if (open_files.find(key) == open_files.end())
+    auto file_iterator = open_files.find(key);
+    if (file_iterator == open_files.end())
     {
         int open_flag = O_WRONLY | O_CREAT;
         if (offset < 0)
@@ -612,7 +614,7 @@ PHP_FUNCTION(swoole_async_write)
     }
     else
     {
-        fd = open_files[key];
+        fd = file_iterator->second;
     }
 
     if (offset < 0)
@@ -905,7 +907,7 @@ PHP_FUNCTION(swoole_async_set)
     if (php_swoole_array_get_value(vht, "display_errors", v))
     {
         convert_to_boolean(v);
-        SWOOLE_G(display_errors) = 0;
+        SWOOLE_G(display_errors) = Z_BVAL_P(v);
     }
     if (php_swoole_array_get_value(vht, "socket_dontwait", v))
     {
@@ -1035,11 +1037,10 @@ static int process_stream_onRead(swReactor *reactor, swEvent *event)
     if (ret > 0)
     {
         ps->buffer->length += ret;
-        if (ps->buffer->length == ps->buffer->size)
+        if (ps->buffer->length == ps->buffer->size && swString_extend(ps->buffer, ps->buffer->size * 2) == 0)
         {
-            swString_extend(ps->buffer, ps->buffer->size * 2);
+            return SW_OK;
         }
-        return SW_OK;
     }
     else if (ret < 0)
     {
@@ -1218,7 +1219,7 @@ PHP_FUNCTION(swoole_async_dns_lookup_coro)
     {
         context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
     }
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 #endif
