@@ -30,8 +30,8 @@
 #define SW_ERROR_MSG_SIZE          1024
 #define SW_MAX_FILE_CONTENT        (64*1024*1024) //for swoole_file_get_contents
 #define SW_MAX_LISTEN_PORT         60000
+#define SW_MAX_CONNECTION          10000
 #define SW_MAX_CONCURRENT_TASK     1024
-#define SW_MAX_CORO_NESTING_LEVEL  128
 #define SW_STACK_BUFFER_SIZE       65536
 
 #ifdef HAVE_MALLOC_TRIM
@@ -52,7 +52,11 @@
 
 #define SW_SOCKET_OVERFLOW_WAIT    100
 #define SW_SOCKET_MAX_DEFAULT      65536
-#define SW_SOCKET_BUFFER_SIZE      (8*1024*1024)
+#ifdef __MACH__
+#define SW_SOCKET_BUFFER_SIZE      262144
+#else
+#define SW_SOCKET_BUFFER_SIZE      8388608
+#endif
 #define SW_SYSTEMD_FDS_START       3
 
 #define SW_GLOBAL_MEMORY_PAGESIZE  (1024*1024*2) //全局内存的分页
@@ -76,7 +80,7 @@
 //#define SW_CLIENT_SOCKET_WAIT
 
 //!!!Don't modify.----------------------------------------------------------
-#if __MACH__
+#ifdef __MACH__
 #define SW_IPC_MAX_SIZE            2048  //MacOS
 #else
 #define SW_IPC_MAX_SIZE            8192  //for IPC, dgram and message-queue max size
@@ -234,13 +238,14 @@
 /**
  * HTTP2 Protocol
  */
-#define SW_HTTP2_DATA_BUFFER_SIZE            8192
-#define SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE   (1 << 12)
-#define SW_HTTP2_MAX_MAX_CONCURRENT_STREAMS  128
-#define SW_HTTP2_MAX_MAX_FRAME_SIZE          ((1u << 14))
-#define SW_HTTP2_MAX_WINDOW_SIZE             ((1u << 31) - 1)
-#define SW_HTTP2_DEFAULT_WINDOW_SIZE         65535
-#define SW_HTTP2_MAX_MAX_HEADER_LIST_SIZE    UINT32_MAX
+#define SW_HTTP2_DATA_BUFFER_SIZE              8192
+#define SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE     (1 << 12)
+#define SW_HTTP2_MAX_MAX_CONCURRENT_STREAMS    128
+#define SW_HTTP2_MAX_MAX_FRAME_SIZE            ((1u << 14))
+#define SW_HTTP2_MAX_WINDOW_SIZE               ((1u << 31) - 1)
+#define SW_HTTP2_DEFAULT_WINDOW_SIZE           65535
+#define SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE  SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE
+#define SW_HTTP2_MAX_MAX_HEADER_LIST_SIZE      UINT32_MAX
 
 #define SW_HTTP_CLIENT_USERAGENT         "swoole-http-client"
 #define SW_HTTP_CLIENT_BOUNDARY_PREKEY   "----SwooleBoundary"
@@ -268,7 +273,8 @@
 #define SW_DEFAULT_MAX_CORO_NUM          3000
 #define SW_DEFAULT_STACK_SIZE            8192
 #define SW_DEFAULT_C_STACK_SIZE          (1024 * 1024 * 2)
-#define SW_MAX_CORO_NUM_LIMIT            0x80000
+#define SW_MAX_CORO_NUM_LIMIT            9223372036854775807LL
+#define SW_MAX_CORO_NESTING_LEVEL        128
 
 #ifdef SW_DEBUG
 #ifndef SW_LOG_TRACE_OPEN
