@@ -42,9 +42,9 @@ typedef struct
 
 typedef struct
 {
-    int current_cid;
+    std::unordered_map<long, Coroutine*> *_map;
+    std::unordered_map<long, Coroutine*>::iterator _cursor;
     int index;
-    int count;
 } coroutine_iterator;
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_void, 0, 0, 0)
@@ -56,6 +56,7 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_create, 0, 0, 1)
     ZEND_ARG_INFO(0, func)
+    ZEND_ARG_VARIADIC_INFO(0, params)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exec, 0, 0, 1)
@@ -90,6 +91,10 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_gethostbyname, 0, 0, 1)
     ZEND_ARG_INFO(0, family)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_defer, 0, 0, 1)
+    ZEND_ARG_INFO(0, callback)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_getaddrinfo, 0, 0, 1)
     ZEND_ARG_INFO(0, hostname)
     ZEND_ARG_INFO(0, family)
@@ -145,7 +150,7 @@ static PHP_METHOD(swoole_coroutine_iterator, __destruct);
 static PHP_METHOD(swoole_exit_exception, getFlags);
 static PHP_METHOD(swoole_exit_exception, getStatus);
 
-static std::unordered_map<int, php_context *> defer_coros;
+static std::unordered_map<int, Coroutine *> user_yield_coros;
 
 static zend_class_entry swoole_coroutine_util_ce;
 static zend_class_entry *swoole_coroutine_util_class_entry_ptr;
@@ -156,16 +161,16 @@ static zend_class_entry *swoole_coroutine_iterator_class_entry_ptr;
 static zend_class_entry swoole_exit_exception_ce;
 static zend_class_entry *swoole_exit_exception_class_entry_ptr;
 
-extern "C"
-{
-int swoole_coroutine_statvfs(const char *path, struct statvfs *buf);
-}
+BEGIN_EXTERN_C()
+extern int swoole_coroutine_statvfs(const char *path, struct statvfs *buf);
+END_EXTERN_C()
 
 static const zend_function_entry swoole_coroutine_util_methods[] =
 {
     ZEND_FENTRY(create, ZEND_FN(swoole_coroutine_create), arginfo_swoole_coroutine_create, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     ZEND_FENTRY(exec, ZEND_FN(swoole_coroutine_exec), arginfo_swoole_coroutine_exec, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     ZEND_FENTRY(gethostbyname, ZEND_FN(swoole_coroutine_gethostbyname), arginfo_swoole_coroutine_gethostbyname, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    ZEND_FENTRY(defer, ZEND_FN(swoole_coroutine_defer), arginfo_swoole_coroutine_defer, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, set, arginfo_swoole_coroutine_set, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, yield, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_MALIAS(swoole_coroutine_util, suspend, yield, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -274,6 +279,8 @@ static int coro_exit_handler(zend_execute_data *execute_data)
 
 void swoole_coroutine_util_init(int module_number)
 {
+    coro_init();
+
     SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_util_ce, "swoole_coroutine", "Swoole\\Coroutine", swoole_coroutine_util_methods);
     swoole_coroutine_util_class_entry_ptr = zend_register_internal_class(&swoole_coroutine_util_ce);
 
@@ -300,6 +307,11 @@ void swoole_coroutine_util_init(int module_number)
 
     SWOOLE_DEFINE(DEFAULT_MAX_CORO_NUM);
     SWOOLE_DEFINE(MAX_CORO_NUM_LIMIT);
+    SWOOLE_DEFINE(MAX_CORO_NESTING_LEVEL);
+    SWOOLE_DEFINE(CORO_INIT);
+    SWOOLE_DEFINE(CORO_WAITING);
+    SWOOLE_DEFINE(CORO_RUNNING);
+    SWOOLE_DEFINE(CORO_END);
 
     //prohibit exit in coroutine
     INIT_CLASS_ENTRY(swoole_exit_exception_ce, "Swoole\\ExitException", swoole_exit_exception_methods);
@@ -329,17 +341,15 @@ static PHP_METHOD(swoole_exit_exception, getStatus)
  */
 static PHP_METHOD(swoole_coroutine_util, yield)
 {
-    int cid = sw_get_current_cid();
-    if (cid < 0)
+    Coroutine* co = coroutine_get_current();
+    if (unlikely(!co))
     {
         swoole_php_fatal_error(E_ERROR, "can not yield outside coroutine");
         RETURN_FALSE;
     }
-
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
-    defer_coros[cid] = context;
-    coro_save(context);
-    coro_yield();
+    user_yield_coros[co->get_cid()] = co;
+    co->yield();
+    RETURN_TRUE;
 }
 
 static PHP_METHOD(swoole_coroutine_util, set)
@@ -389,42 +399,28 @@ static PHP_METHOD(swoole_coroutine_util, set)
 
 PHP_FUNCTION(swoole_coroutine_create)
 {
-    zval *callback;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &callback) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    zend_fcall_info fci = empty_fcall_info;
+    zend_fcall_info_cache fci_cache = empty_fcall_info_cache;
+
+    ZEND_PARSE_PARAMETERS_START(1, -1)
+        Z_PARAM_FUNC(fci, fci_cache)
+        Z_PARAM_VARIADIC('*', fci.params, fci.param_count)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     if (unlikely(SWOOLE_G(req_status) == PHP_SWOOLE_CALL_USER_SHUTDOWNFUNC_BEGIN))
     {
         zend_function *func = (zend_function *) EG(current_execute_data)->prev_execute_data->func;
-        if (memcmp(ZSTR_VAL(func->common.function_name), ZEND_STRS("__destruct")) == 0)
+        if (unlikely(memcmp(ZSTR_VAL(func->common.function_name), ZEND_STRS("__destruct")) == 0))
         {
             swoole_php_fatal_error(E_ERROR, "can not use coroutine in __destruct after php_request_shutdown");
             RETURN_FALSE;
         }
     }
-    char *func_name = NULL;
-    zend_fcall_info_cache *func_cache = ( zend_fcall_info_cache *) emalloc(sizeof(zend_fcall_info_cache));
-    if (!sw_zend_is_callable_ex(callback, NULL, 0, &func_name, NULL, func_cache, NULL))
-    {
-        swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
-        efree(func_name);
-        return;
-    }
-    efree(func_name);
-    if (COROG.active == 0)
-    {
-        coro_init();
-    }
+
     php_swoole_check_reactor();
-    callback = sw_zval_dup(callback);
-    Z_TRY_ADDREF_P(callback);
 
     zval *retval = NULL;
-    zval *args[1];
-    int cid = coro_create(func_cache, args, 0, &retval, NULL, NULL);
-    sw_zval_free(callback);
-    efree(func_cache);
+    long cid = sw_coro_create(&fci_cache, fci.param_count, fci.params, retval);
     if (EG(exception))
     {
         zend_exception_error(EG(exception), E_ERROR);
@@ -433,13 +429,13 @@ PHP_FUNCTION(swoole_coroutine_create)
     {
         zval_ptr_dtor(retval);
     }
-    if (cid < 0)
+    if (likely(cid > 0))
     {
-        RETURN_FALSE;
+        RETURN_LONG(cid);
     }
     else
     {
-        RETURN_LONG(cid);
+        RETURN_FALSE;
     }
 }
 
@@ -451,33 +447,16 @@ static PHP_METHOD(swoole_coroutine_util, resume)
         RETURN_FALSE;
     }
 
-    if (defer_coros.find(cid) == defer_coros.end())
+    auto coroutine_iterator = user_yield_coros.find(cid);
+    if (coroutine_iterator == user_yield_coros.end())
     {
-        swoole_php_fatal_error(E_WARNING, "no coroutine can resume.");
+        swoole_php_fatal_error(E_WARNING, "you can not resume the coroutine which is in IO operation.");
         RETURN_FALSE;
     }
 
-    php_context *context = defer_coros[cid];
-    defer_coros.erase(cid);
-
-    zend_vm_stack origin_vm_stack = EG(vm_stack);
-    zval *origin_vm_stack_top = EG(vm_stack_top);
-    zval *origin_vm_stack_end = EG(vm_stack_end);
-
-    zval *retval = NULL;
-    zval *result;
-    SW_MAKE_STD_ZVAL(result);
-    ZVAL_BOOL(result, 1);
-    int ret = coro_resume(context, result, &retval);
-    if (ret == CORO_END && retval)
-    {
-        zval_ptr_dtor(retval);
-    }
-    zval_ptr_dtor(result);
-    efree(context);
-    EG(vm_stack) = origin_vm_stack;
-    EG(vm_stack_top) = origin_vm_stack_top;
-    EG(vm_stack_end) = origin_vm_stack_end;
+    Coroutine* co = coroutine_iterator->second;
+    user_yield_coros.erase(cid);
+    co->resume();
     RETURN_TRUE;
 }
 
@@ -528,8 +507,7 @@ static PHP_METHOD(swoole_coroutine_util, sleep)
     }
 
     php_swoole_check_reactor();
-
-    swoole_coroutine_sleep(seconds);
+    Coroutine::sleep(seconds);
     RETURN_TRUE;
 }
 
@@ -550,7 +528,7 @@ static void aio_onReadCompleted(swAio_event *event)
     }
 
     php_context *context = (php_context *) event->object;
-    int ret = coro_resume(context, result, &retval);
+    int ret = sw_coro_resume(context, result, retval);
     if (ret == CORO_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -585,7 +563,7 @@ static void aio_onFgetsCompleted(swAio_event *event)
         stream->eof = 1;
     }
 
-    int ret = coro_resume(context, result, &retval);
+    int ret = sw_coro_resume(context, result, retval);
     if (ret == CORO_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -611,7 +589,7 @@ static void aio_onWriteCompleted(swAio_event *event)
     }
 
     php_context *context = (php_context *) event->object;
-    int ret = coro_resume(context, result, &retval);
+    int ret = sw_coro_resume(context, result, retval);
     if (ret == CORO_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -654,7 +632,7 @@ static int co_socket_onReadable(swReactor *reactor, swEvent *event)
         sock->buf->len = n;
         ZVAL_STR(&result, sock->buf);
     }
-    int ret = coro_resume(context, &result, &retval);
+    int ret = sw_coro_resume(context, &result, retval);
     zval_ptr_dtor(&result);
     if (ret == CORO_END && retval)
     {
@@ -690,7 +668,7 @@ static int co_socket_onWritable(swReactor *reactor, swEvent *event)
     {
         ZVAL_LONG(&result, n);
     }
-    int ret = coro_resume(context, &result, &retval);
+    int ret = sw_coro_resume(context, &result, retval);
     zval_ptr_dtor(&result);
     if (ret == CORO_END && retval)
     {
@@ -726,8 +704,8 @@ static void co_socket_read(int fd, zend_long length, INTERNAL_FUNCTION_PARAMETER
 
     sock->context.state = SW_CORO_CONTEXT_RUNNING;
 
-    coro_save(&sock->context);
-    coro_yield();
+    sw_coro_save(return_value, &sock->context);
+    sw_coro_yield();
 }
 
 static void co_socket_write(int fd, char* str, size_t l_str, INTERNAL_FUNCTION_PARAMETERS)
@@ -764,8 +742,8 @@ static void co_socket_write(int fd, char* str, size_t l_str, INTERNAL_FUNCTION_P
 
     sock->nbytes = l_str;
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, fread)
@@ -853,8 +831,8 @@ static PHP_METHOD(swoole_coroutine_util, fread)
 
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, fgets)
@@ -937,8 +915,8 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
     context->coro_params = *handle;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, fwrite)
@@ -1015,8 +993,8 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
 
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, readFile)
@@ -1035,7 +1013,7 @@ static PHP_METHOD(swoole_coroutine_util, readFile)
 
     php_swoole_check_aio();
 
-    swString *result = swoole_coroutine_read_file(filename, flags & LOCK_EX);
+    swString *result = Coroutine::read_file(filename, flags & LOCK_EX);
     if (result == NULL)
     {
         RETURN_FALSE;
@@ -1064,12 +1042,6 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
         Z_PARAM_LONG(flags)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    swAio_event ev;
-    bzero(&ev, sizeof(swAio_event));
-
-    ev.nbytes = l_data;
-    ev.buf = data;
-
     int _flags = O_CREAT | O_WRONLY;
     if (flags & PHP_FILE_APPEND)
     {
@@ -1080,7 +1052,7 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
         _flags |= O_TRUNC;
     }
 
-    ssize_t retval = swoole_coroutine_write_file(filename, data, l_data, flags & LOCK_EX, _flags);
+    ssize_t retval = Coroutine::write_file(filename, data, l_data, flags & LOCK_EX, _flags);
     if (retval < 0)
     {
         RETURN_FALSE
@@ -1110,7 +1082,7 @@ static void coro_dns_onResolveCompleted(swAio_event *event)
         ZVAL_BOOL(result, 0);
     }
 
-    int ret = coro_resume(context, result, &retval);
+    int ret = sw_coro_resume(context, result, retval);
     if (ret == CORO_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -1165,7 +1137,7 @@ static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
         SwooleG.error = req->error;
     }
 
-    int ret = coro_resume(context, result, &retval);
+    int ret = sw_coro_resume(context, result, retval);
     if (ret == CORO_END && retval)
     {
         zval_ptr_dtor(retval);
@@ -1243,8 +1215,8 @@ PHP_FUNCTION(swoole_coroutine_gethostbyname)
         RETURN_FALSE;
     }
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
@@ -1318,8 +1290,8 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
         RETURN_FALSE;
     }
 
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, getBackTrace)
@@ -1344,7 +1316,7 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
             RETURN_FALSE;
         }
         zend_execute_data *ex_backup = EG(current_execute_data);
-        EG(current_execute_data) = task->yield_execute_data;
+        EG(current_execute_data) = task->execute_data;
         zend_fetch_debug_backtrace(return_value, 0, options, limit);
         EG(current_execute_data) = ex_backup;
     }
@@ -1353,38 +1325,28 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
 static PHP_METHOD(swoole_coroutine_iterator, rewind)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    bzero(itearator, sizeof(coroutine_iterator));
-    itearator->count = COROG.coro_num;
+    itearator->_cursor = itearator->_map->begin();
+    itearator->index = 0;
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, valid)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    int cid = itearator->current_cid;
-
-    for (; itearator->count > 0 && cid < MAX_CORO_NUM_LIMIT + 1; cid++)
-    {
-        if (coroutine_get_by_id(cid))
-        {
-            itearator->current_cid = cid;
-            itearator->index++;
-            itearator->count--;
-            RETURN_TRUE;
-        }
-    }
-    RETURN_FALSE;
+    RETURN_BOOL(itearator->_cursor != itearator->_map->end());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, current)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    RETURN_LONG(itearator->current_cid);
+    Coroutine *co = itearator->_cursor->second;
+    RETURN_LONG(co->get_cid());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, next)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    itearator->current_cid++;
+    itearator->_cursor++;
+    itearator->index++;
 }
 
 PHP_METHOD(swoole_coroutine_iterator, key)
@@ -1408,8 +1370,9 @@ static PHP_METHOD(swoole_coroutine_iterator, __destruct)
 static PHP_METHOD(swoole_coroutine_util, listCoroutines)
 {
     object_init_ex(return_value, swoole_coroutine_iterator_class_entry_ptr);
-    coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
+    coroutine_iterator *itearator = (coroutine_iterator *) emalloc(sizeof(coroutine_iterator));
     bzero(itearator, sizeof(coroutine_iterator));
+    itearator->_map = coroutine_get_map();
     swoole_set_object(return_value, itearator);
 }
 
@@ -1526,3 +1489,30 @@ PHP_FUNCTION(swoole_coroutine_exec)
 
     swString_free(buffer);
 }
+
+PHP_FUNCTION(swoole_coroutine_defer)
+{
+    coro_check();
+
+    zval *callback;
+    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &callback) == FAILURE)
+    {
+        RETURN_FALSE;
+    }
+
+    char *func_name;
+    if (!sw_zend_is_callable(callback, 0, &func_name))
+    {
+        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
+        efree(func_name);
+        RETURN_FALSE;
+    }
+    efree(func_name);
+
+    php_defer_callback *defer = (php_defer_callback *) emalloc(sizeof(php_defer_callback));
+    defer->callback = &defer->_callback;
+    memcpy(defer->callback, callback, sizeof(zval));
+    Z_TRY_ADDREF_P(callback);
+
+    sw_coro_add_defer_task(php_swoole_event_onDefer, defer);
+}
