@@ -17,9 +17,6 @@
  */
 
 #include "php_swoole.h"
-#ifdef SW_COROUTINE
-#include "swoole_coroutine.h"
-#endif
 
 typedef struct
 {
@@ -34,16 +31,9 @@ typedef struct
     zval *socket;
 } php_reactor_fd;
 
-typedef struct
-{
-    zval _callback;
-    zval *callback;
-} php_defer_callback;
-
 static int php_swoole_event_onRead(swReactor *reactor, swEvent *event);
 static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event);
 static int php_swoole_event_onError(swReactor *reactor, swEvent *event);
-static void php_swoole_event_onDefer(void *_cb);
 static void php_swoole_event_onEndCallback(void *_cb);
 
 static void free_event_callback(void* data)
@@ -133,7 +123,6 @@ static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
 
 static int php_swoole_event_onError(swReactor *reactor, swEvent *event)
 {
-
     int error;
     socklen_t len = sizeof(error);
 
@@ -155,14 +144,14 @@ static int php_swoole_event_onError(swReactor *reactor, swEvent *event)
     return SW_OK;
 }
 
-static void php_swoole_event_onDefer(void *_cb)
+void php_swoole_event_onDefer(void *_cb)
 {
     php_defer_callback *defer = _cb;
-
     zval *retval;
-    if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL) == FAILURE)
+
+    if (sw_call_user_function_anyway(NULL, defer->callback, &retval, 0, NULL, 0) == FAILURE)
     {
-        swoole_php_fatal_error(E_WARNING, "swoole_event: defer handler error");
+        swoole_php_fatal_error(E_WARNING, "defer callback handler error.");
         return;
     }
     if (EG(exception))
@@ -180,11 +169,11 @@ static void php_swoole_event_onDefer(void *_cb)
 static void php_swoole_event_onEndCallback(void *_cb)
 {
     php_defer_callback *defer = _cb;
-
     zval *retval;
+
     if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL) == FAILURE)
     {
-        swoole_php_fatal_error(E_WARNING, "swoole_event: defer handler error");
+        swoole_php_fatal_error(E_WARNING, "swoole_event: cycle callback handler error.");
         return;
     }
     if (EG(exception))
@@ -272,22 +261,16 @@ void php_swoole_event_wait()
             swSignalfd_setup(SwooleG.main_reactor);
         }
 #endif
-
-#ifdef SW_COROUTINE
-        if (COROG.active == 0)
-        {
-            coro_init();
-        }
-#endif
         if (!swReactor_empty(SwooleG.main_reactor))
         {
+            SW_DECLARE_EG_SCOPE(scope);
             SW_SAVE_EG_SCOPE(scope);
             int ret = SwooleG.main_reactor->wait(SwooleG.main_reactor, NULL);
             if (ret < 0)
             {
                 swoole_php_fatal_error(E_ERROR, "reactor wait failed. Error: %s [%d]", strerror(errno), errno);
             }
-            SW_RESUME_EG_SCOPE(scope);
+            SW_SET_EG_SCOPE(scope);
         }
         if (SwooleG.timer.map)
         {
@@ -489,7 +472,7 @@ PHP_FUNCTION(swoole_event_add)
     {
         if (!sw_zend_is_callable(cb_read, 0, &func_name))
         {
-            swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+            swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
             efree(func_name);
             RETURN_FALSE;
         }
@@ -507,7 +490,7 @@ PHP_FUNCTION(swoole_event_add)
     {
         if (!sw_zend_is_callable(cb_write, 0, &func_name))
         {
-            swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+            swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
             efree(func_name);
             RETURN_FALSE;
         }
@@ -612,7 +595,7 @@ PHP_FUNCTION(swoole_event_set)
     {
         if (!sw_zend_is_callable(cb_read, 0, &func_name))
         {
-            swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+            swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
             efree(func_name);
             RETURN_FALSE;
         }
@@ -638,7 +621,7 @@ PHP_FUNCTION(swoole_event_set)
         }
         if (!sw_zend_is_callable(cb_write, 0, &func_name))
         {
-            swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+            swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
             efree(func_name);
             RETURN_FALSE;
         }
@@ -721,7 +704,7 @@ PHP_FUNCTION(swoole_event_defer)
     char *func_name;
     if (!sw_zend_is_callable(callback, 0, &func_name))
     {
-        swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
         efree(func_name);
         RETURN_FALSE;
     }
@@ -770,7 +753,7 @@ PHP_FUNCTION(swoole_event_cycle)
     char *func_name;
     if (!sw_zend_is_callable(callback, 0, &func_name))
     {
-        swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
         efree(func_name);
         RETURN_FALSE;
     }
@@ -837,13 +820,6 @@ PHP_FUNCTION(swoole_event_dispatch)
     }
 #endif
 
-#ifdef SW_COROUTINE
-    if (COROG.active == 0)
-    {
-        coro_init();
-    }
-#endif
-
     int ret = SwooleG.main_reactor->wait(SwooleG.main_reactor, NULL);
     if (ret < 0)
     {
