@@ -233,6 +233,13 @@ static int http_client_coro_execute(zval *zobject, http_client_coro_property *hc
     if (!hcc->socket)
     {
         hcc->socket = new Socket(SW_SOCK_TCP);
+        if (unlikely(hcc->socket->socket == nullptr))
+        {
+            delete hcc->socket;
+            swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
+            zend_update_property_long(Z_OBJCE_P(zobject), zobject, ZEND_STRL("errCode"), errno);
+            return SW_ERR;
+        }
         zval *ztmp;
         HashTable *vht;
         zval *zset = sw_zend_read_property(Z_OBJCE_P(zobject), zobject, ZEND_STRL("setting"), 1);
@@ -1066,7 +1073,7 @@ static PHP_METHOD(swoole_http_client_coro, recv)
 {
     http_client_coro_property *hcc = (http_client_coro_property *) swoole_get_property(getThis(), 0);
     http_client *http = (http_client *) swoole_get_object(getThis());
-    double timeout = http->timeout;
+    double timeout;
 
     if (!http)
     {
@@ -1074,6 +1081,10 @@ static PHP_METHOD(swoole_http_client_coro, recv)
         zend_update_property_long(swoole_http_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         RETURN_FALSE;
     }
+    else
+    {
+        timeout = http->timeout;
+    }
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
