@@ -113,12 +113,13 @@ static sw_inline void http2_add_header(nghttp2_nv *headers, const char *k, int k
 static int http_build_trailer(http_context *ctx, uchar *buffer)
 {
     int ret;
-    nghttp2_nv nv[128];
     size_t index = 0;
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 0);
+    uint32_t nv_size = ZVAL_IS_ARRAY(ztrailer) ? php_swoole_array_length(ztrailer) : 0;
 
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 1);
-    if (ZVAL_IS_ARRAY(ztrailer))
+    if (nv_size > 0)
     {
+        nghttp2_nv *nv = (nghttp2_nv *) ecalloc(sizeof(nghttp2_nv), nv_size);
         HashTable *ht = Z_ARRVAL_P(ztrailer);
         zval *value = NULL;
         char *key = NULL;
@@ -134,47 +135,44 @@ static int http_build_trailer(http_context *ctx, uchar *buffer)
             (void) type;
         }
         SW_HASHTABLE_FOREACH_END();
-    }
 
-    ssize_t rv;
-    size_t buflen;
-    size_t i;
-    size_t sum = 0;
-    http2_session *client = http2_sessions[ctx->fd];
-    nghttp2_hd_deflater *deflater = client->deflater;
+        ssize_t rv;
+        size_t buflen;
+        http2_session *client = http2_sessions[ctx->fd];
+        nghttp2_hd_deflater *deflater = client->deflater;
 
-    if (!deflater)
-    {
-        ret = nghttp2_hd_deflate_new(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
-        if (ret != 0)
+        if (!deflater)
         {
-            swoole_php_error(E_WARNING, "nghttp2_hd_deflate_init failed with error: %s\n", nghttp2_strerror(ret));
+            ret = nghttp2_hd_deflate_new(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
+            if (ret != 0)
+            {
+                swoole_php_error(E_WARNING, "nghttp2_hd_deflate_init() failed with error: %s\n", nghttp2_strerror(ret));
+                return SW_ERR;
+            }
+            client->deflater = deflater;
+        }
+
+        buflen = nghttp2_hd_deflate_bound(deflater, nv, index);
+        if (buflen > SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE)
+        {
+            swoole_php_error(E_WARNING, "header cannot bigger than remote max_header_list_size %u.", SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE);
+            efree(nv);
             return SW_ERR;
         }
-        client->deflater = deflater;
+        rv = nghttp2_hd_deflate_hd(deflater, (uchar *) buffer, buflen, nv, index);
+        if (rv < 0)
+        {
+            swoole_php_error(E_WARNING, "nghttp2_hd_deflate_hd() failed with error: %s\n", nghttp2_strerror((int ) rv));
+            efree(nv);
+            return SW_ERR;
+        }
+
+        efree(nv);
+
+        return rv;
     }
 
-    for (i = 0; i < index; ++i)
-    {
-        sum += nv[i].namelen + nv[i].valuelen;
-    }
-
-    buflen = nghttp2_hd_deflate_bound(deflater, nv, index);
-    rv = nghttp2_hd_deflate_hd(deflater, (uchar *) buffer, buflen, nv, index);
-    if (rv < 0)
-    {
-        swoole_php_error(E_WARNING, "nghttp2_hd_deflate_hd() failed with error: %s\n", nghttp2_strerror((int ) rv));
-        return SW_ERR;
-    }
-
-    ret = nghttp2_hd_deflate_change_table_size(deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
-    if (ret != 0)
-    {
-        swoole_php_error(E_WARNING, "nghttp2_hd_deflate_change_table_size failed with error: %s\n", nghttp2_strerror(ret));
-        return SW_ERR;
-    }
-
-    return rv;
+    return SW_OK;
 }
 
 static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
@@ -187,17 +185,17 @@ static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
     SW_SEPARATE_ZVAL(zrequest_object);
     SW_SEPARATE_ZVAL(zresponse_object);
 
+    zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, from_fd, SW_SERVER_CB_onRequest);
+    zval args[2];
+    args[0] = *zrequest_object;
+    args[1] = *zresponse_object;
+
     if (SwooleG.enable_coroutine)
     {
-        zval *args[2];
-        args[0] = zrequest_object;
-        args[1] = zresponse_object;
-
-        zend_fcall_info_cache *cache = php_swoole_server_get_cache(serv, from_fd, SW_SERVER_CB_onRequest);
-        int ret = coro_create(cache, args, 2, &retval, NULL, NULL);
+        long ret = sw_coro_create(fci_cache, 2, args, retval);
         if (ret < 0)
         {
-            if (ret == CORO_LIMIT)
+            if (ret < 0)
             {
                 serv->factory.end(&SwooleG.serv->factory, fd);
             }
@@ -206,13 +204,7 @@ static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
     }
     else
     {
-        zval args[2];
-        args[0] = *zrequest_object;
-        args[1] = *zresponse_object;
-
-        zval *zcallback = php_swoole_server_get_callback(serv, from_fd, SW_SERVER_CB_onRequest);
-        zend_fcall_info_cache *fci_cache = php_swoole_server_get_cache(serv, from_fd, SW_SERVER_CB_onRequest);
-        if (sw_call_user_function_fast_ex(zcallback, fci_cache, &retval, 2, args) == FAILURE)
+        if (sw_call_user_function_fast_ex(NULL, fci_cache, &retval, 2, args) == FAILURE)
         {
             swoole_php_error(E_WARNING, "Http2 onRequest handler error");
         }
@@ -239,7 +231,8 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
     char intbuf[2][16];
     int ret;
     size_t index = 0;
-    nghttp2_nv nv[128];
+    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
+    nghttp2_nv *nv = (nghttp2_nv *) ecalloc(sizeof(nghttp2_nv), 8 + (ZVAL_IS_ARRAY(zheader) ? php_swoole_array_length(zheader) : 0));
 
     assert(ctx->send_header == 0);
 
@@ -252,11 +245,9 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
     http2_add_header(&nv[index++], ZEND_STRL(":status"), intbuf[0], ret);
 
     // headers
-    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
     if (ZVAL_IS_ARRAY(zheader))
     {
         uint32_t header_flag = 0x0;
-
         HashTable *ht = Z_ARRVAL_P(zheader);
         zval *value = NULL;
         char *key = NULL;
@@ -285,6 +276,10 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
             {
                 header_flag |= HTTP_HEADER_CONTENT_TYPE;
             }
+            if (ZVAL_IS_NULL(value))
+            {
+                continue;
+            }
             http2_add_header(&nv[index++], key, keylen, Z_STRVAL_P(value), Z_STRLEN_P(value));
         }
         SW_HASHTABLE_FOREACH_END();
@@ -347,38 +342,40 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
 
     ctx->send_header = 1;
 
-    ssize_t rv;
+    ssize_t rv = SW_ERR;
     size_t buflen;
-    size_t i;
-    size_t sum = 0;
-
-    nghttp2_hd_deflater *deflater;
-    ret = nghttp2_hd_deflate_new(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
-    if (ret != 0)
+    http2_session *client = http2_sessions[ctx->fd];
+    nghttp2_hd_deflater *deflater = client->deflater;
+    if (!deflater)
     {
-        swoole_php_error(E_WARNING, "nghttp2_hd_deflate_init failed with error: %s\n", nghttp2_strerror(ret));
-        return SW_ERR;
-    }
-
-    for (i = 0; i < index; ++i)
-    {
-        sum += nv[i].namelen + nv[i].valuelen;
+        ret = nghttp2_hd_deflate_new(&deflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
+        if (ret != 0)
+        {
+            swoole_php_error(E_WARNING, "nghttp2_hd_deflate_init() failed with error: %s\n", nghttp2_strerror(ret));
+            goto _error;
+        }
+        client->deflater = deflater;
     }
 
     buflen = nghttp2_hd_deflate_bound(deflater, nv, index);
+    if (buflen > SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE)
+    {
+        swoole_php_error(E_WARNING, "header cannot bigger than remote max_header_list_size %u.", SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE);
+        goto _error;
+    }
     rv = nghttp2_hd_deflate_hd(deflater, (uchar *) buffer, buflen, nv, index);
     if (rv < 0)
     {
         swoole_php_error(E_WARNING, "nghttp2_hd_deflate_hd() failed with error: %s\n", nghttp2_strerror((int ) rv));
-        return SW_ERR;
+        goto _error;
     }
 
+    _error:
     if (date_str)
     {
         efree(date_str);
     }
-
-    nghttp2_hd_deflate_del(deflater);
+    efree(nv);
 
     return rv;
 }
@@ -401,6 +398,11 @@ int swoole_http2_do_response(http_context *ctx, swString *body)
 #endif
 
     ret = http2_build_header(ctx, (uchar *) header_buffer, body->length);
+    if (ret < 0)
+    {
+        return SW_ERR;
+    }
+
     swString_clear(swoole_http_buffer);
 
     /**
@@ -675,11 +677,6 @@ static int http2_parse_header(http2_session *client, http_context *ctx, int flag
         }
     }
 
-    rv = nghttp2_hd_inflate_change_table_size(inflater, SW_HTTP2_DEFAULT_HEADER_TABLE_SIZE);
-    if (rv != 0)
-    {
-        return rv;
-    }
     return SW_OK;
 }
 
@@ -821,13 +818,14 @@ int swoole_http2_onFrame(swConnection *conn, swEventData *req)
     case SW_HTTP2_TYPE_DATA:
     {
         swHttp2FrameTraceLog(recv, "data");
-        if (client->streams.find(stream_id) == client->streams.end())
+        auto stream_iterator = client->streams.find(stream_id);
+        if (stream_iterator == client->streams.end())
         {
             zval_ptr_dtor(zdata);
             swoole_error_log(SW_LOG_WARNING, SW_ERROR_HTTP2_STREAM_NOT_FOUND, "http2 stream#%d not found.", stream_id);
             return SW_ERR;
         }
-        stream = client->streams[stream_id];
+        stream = stream_iterator->second;
         ctx = stream->ctx;
 
         zrequest_object = ctx->request.zobject;
@@ -873,7 +871,7 @@ int swoole_http2_onFrame(swConnection *conn, swEventData *req)
                 size_t n = multipart_parser_execute(multipart_parser, buffer->str, buffer->length);
                 if (n != (size_t) length)
                 {
-                    swoole_php_fatal_error(E_WARNING, "parse multipart body failed.");
+                    swoole_error_log(SW_LOG_WARNING, SW_ERROR_SERVER_INVALID_REQUEST, "parse multipart body failed, n=%zu.", n);
                 }
             }
             http2_onRequest(ctx, from_fd);
@@ -943,11 +941,12 @@ int swoole_http2_onFrame(swConnection *conn, swEventData *req)
 
 void swoole_http2_free(swConnection *conn)
 {
-    if (http2_sessions.find(conn->session_id) == http2_sessions.end())
+    auto session_iterator = http2_sessions.find(conn->session_id);
+    if (session_iterator == http2_sessions.end())
     {
         return;
     }
-    http2_session *client = http2_sessions[conn->session_id];
+    http2_session *client = session_iterator->second;
     http2_sessions.erase(conn->session_id);
     delete client;
 }
