@@ -361,7 +361,7 @@ typedef struct _mysql_client
     zend_bool suspending;
     mysql_io_status iowait;
     zval *result;
-    int cid;
+    long cid;
 #endif
     uint8_t state;
     uint32_t switch_check :1; /* check if server request auth switch */
@@ -410,6 +410,14 @@ typedef struct _mysql_client
 #define SW_MYSQL_GROUP_FLAG              32768
 #define SW_MYSQL_NUM_FLAG                32768
 
+#define SW_MYSQL_PACKET_OK   0x0
+#define SW_MYSQL_PACKET_NULL 0xfb
+#define SW_MYSQL_PACKET_EOF  0xfe
+#define SW_MYSQL_PACKET_ERR  0xff
+
+/* int<3>	payload_length + int<1>	sequence_id */
+#define SW_MYSQL_PACKET_HEADER_SIZE  4
+
 #define mysql_uint2korr(A)  (uint16_t) (((uint16_t) ((zend_uchar) (A)[0])) +\
                                ((uint16_t) ((zend_uchar) (A)[1]) << 8))
 #define mysql_uint3korr(A)  (uint32_t) (((uint32_t) ((zend_uchar) (A)[0])) +\
@@ -470,14 +478,34 @@ int mysql_handshake(mysql_connector *connector, char *buf, int len);
 int mysql_parse_auth_signature(swString *buffer, mysql_connector *connector);
 int mysql_parse_rsa(mysql_connector *connector, char *buf, int len);
 int mysql_auth_switch(mysql_connector *connector, char *buf, int len);
-int mysql_request(swString *sql, swString *buffer);
-int mysql_prepare(swString *sql, swString *buffer);
+int mysql_request_pack(swString *sql, swString *buffer);
+int mysql_prepare_pack(swString *sql, swString *buffer);
 int mysql_response(mysql_client *client);
 int mysql_is_over(mysql_client *client);
 
 #ifdef SW_MYSQL_DEBUG
 void mysql_client_info(mysql_client *client);
 void mysql_column_info(mysql_field *field);
+#define swMysqlPacketDump(packet, len, title) \
+    do { \
+        unsigned int of = 0; \
+        swDebug("+----------+------------+-------------------------------------------------------+"); \
+        swDebug("| P#%-6d | L%-9zu | %-10zu %42s |", (packet)[3], mysql_uint3korr(packet), len, title); \
+        swDebug("+----------+------------+-----------+-----------+------------+------------------+"); \
+        for (of = 0; of < len; of += 16) { \
+            char hex[16*3+1]; \
+            char str[16+1]; \
+            int i, hof = 0, sof = 0; \
+            for (i = of ; i < of + 16 && i < len ; i++) { \
+                hof += sprintf(hex+hof, "%02x ", (packet)[i] & 0xff); \
+                sof += sprintf(str+sof, "%c", isprint((int)(packet)[i]) ? (packet)[i] : '.'); \
+            } \
+            swDebug("| %08x | %-48s| %-16s |", of, hex, str); \
+        } \
+        swDebug("+----------+------------+-----------+-----------+------------+------------------+"); \
+    } while(0)
+#else
+#define swMysqlPacketDump(packet, len, title)
 #endif
 
 static sw_inline void mysql_pack_length(int length, char *buf)
