@@ -27,6 +27,7 @@ static PHP_METHOD(swoole_mysql_coro, __destruct);
 static PHP_METHOD(swoole_mysql_coro, connect);
 static PHP_METHOD(swoole_mysql_coro, query);
 static PHP_METHOD(swoole_mysql_coro, recv);
+static PHP_METHOD(swoole_mysql_coro, nextResult);
 #ifdef SW_USE_MYSQLND
 static PHP_METHOD(swoole_mysql_coro, escape);
 #endif
@@ -115,6 +116,7 @@ static const zend_function_entry swoole_mysql_coro_methods[] =
     PHP_ME(swoole_mysql_coro, connect, arginfo_swoole_mysql_coro_connect, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, query, arginfo_swoole_mysql_coro_query, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, recv, arginfo_swoole_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_mysql_coro, nextResult, arginfo_swoole_void, ZEND_ACC_PUBLIC)
 #ifdef SW_USE_MYSQLND
     PHP_ME(swoole_mysql_coro, escape, arginfo_swoole_mysql_coro_escape, ZEND_ACC_PUBLIC)
 #endif
@@ -391,7 +393,7 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
     client->state = SW_MYSQL_STATE_QUERY;
 
     //OK
-    if (client->response.response_type == 0)
+    if (client->response.response_type == SW_MYSQL_PACKET_OK)
     {
         SW_ALLOC_INIT_ZVAL(*result);
         // prepare finished and create statement
@@ -420,7 +422,7 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
         }
     }
     //ERROR
-    else if (client->response.response_type == 255)
+    else if (client->response.response_type == SW_MYSQL_PACKET_ERR)
     {
         SW_ALLOC_INIT_ZVAL(*result);
         ZVAL_FALSE(*result);
@@ -522,7 +524,7 @@ static int swoole_mysql_coro_statement_close(mysql_statement *stmt)
     mysql_request_buffer->length += 4;
     //length
     mysql_pack_length(mysql_request_buffer->length - 4, mysql_request_buffer->str);
-    //send data, mysql-server would not reply
+    //tell sever to close the statement, mysql-server would not reply
     SwooleG.main_reactor->write(SwooleG.main_reactor, stmt->client->fd, mysql_request_buffer->str, mysql_request_buffer->length);
 
     return SW_OK;
@@ -812,8 +814,8 @@ static PHP_METHOD(swoole_mysql_coro, connect)
         connector->timer = swTimer_add(&SwooleG.timer, (int) (connector->timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
     }
     client->cid = sw_get_current_cid();
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_mysql_coro, query)
@@ -851,8 +853,6 @@ static PHP_METHOD(swoole_mysql_coro, query)
         RETURN_FALSE;
     }
 
-    swString_clear(mysql_request_buffer);
-
     if (mysql_query(getThis(), client, &sql, NULL) < 0)
     {
         RETURN_FALSE;
@@ -874,8 +874,41 @@ static PHP_METHOD(swoole_mysql_coro, query)
         RETURN_TRUE;
     }
     client->cid = sw_get_current_cid();
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
+}
+
+static PHP_METHOD(swoole_mysql_coro, nextResult)
+{
+    mysql_client *client = (mysql_client *) swoole_get_object(getThis());
+    if (!client)
+    {
+        RETURN_FALSE;
+    }
+
+    if (client->buffer && (size_t) client->buffer->offset < client->buffer->length)
+    {
+        client->cmd = SW_MYSQL_COM_QUERY;
+        client->state = SW_MYSQL_STATE_READ_START;
+        client->statement = nullptr;
+        zval *result = NULL;
+        if (swoole_mysql_coro_parse_response(client, &result, 1) == SW_OK)
+        {
+            swoole_mysql_coro_parse_end(client, client->buffer); // ending tidy up
+            zval _result = *result;
+            efree(result);
+            result = &_result;
+            RETURN_ZVAL(result, 0, 1);
+        }
+        else
+        {
+            RETURN_FALSE;
+        }
+    }
+    else
+    {
+        RETURN_NULL();
+    }
 }
 
 static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_transaction, zend_execute_data *execute_data, zval *return_value)
@@ -930,9 +963,9 @@ static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_
             client->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
         }
         client->cid = sw_get_current_cid();
-        coro_save(context);
+        sw_coro_save(return_value, context);
         coro_use_return_value();
-        coro_yield();
+        sw_coro_yield();
         // resume true
         if (Z_BVAL_P(return_value))
         {
@@ -1010,8 +1043,8 @@ static PHP_METHOD(swoole_mysql_coro, recv)
     client->suspending = 1;
     client->cid = sw_get_current_cid();
     php_context *context = (php_context *) swoole_get_property(getThis(), 0);
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_mysql_coro, prepare)
@@ -1048,16 +1081,19 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         RETURN_FALSE;
     }
 
+    if (client->buffer)
+    {
+        swString_clear(client->buffer);
+    }
+
     client->cmd = SW_MYSQL_COM_STMT_PREPARE;
     client->state = SW_MYSQL_STATE_READ_START;
 
-    swString_clear(mysql_request_buffer);
-
-    if (mysql_prepare(&sql, mysql_request_buffer) < 0)
+    if (mysql_prepare_pack(&sql, mysql_request_buffer) < 0)
     {
         RETURN_FALSE;
     }
-    //send query
+    //send prepare command
     if (SwooleG.main_reactor->write(SwooleG.main_reactor, client->fd, mysql_request_buffer->str, mysql_request_buffer->length) < 0)
     {
         //connection is closed
@@ -1084,8 +1120,8 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
     }
     client->suspending = 1;
     client->cid = sw_get_current_cid();
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_mysql_coro_statement, execute)
@@ -1112,6 +1148,11 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
         RETURN_FALSE;
     }
 
+    if (stmt->buffer)
+    {
+        swString_clear(stmt->buffer);
+    }
+
     if (swoole_mysql_coro_execute(getThis(), client, params) < 0)
     {
         RETURN_FALSE;
@@ -1133,8 +1174,8 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
     }
     client->suspending = 1;
     client->cid = sw_get_current_cid();
-    coro_save(context);
-    coro_yield();
+    sw_coro_save(return_value, context);
+    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_mysql_coro_statement, fetch)
@@ -1250,7 +1291,7 @@ static PHP_METHOD(swoole_mysql_coro_statement, nextResult)
     }
     else
     {
-        RETURN_NULL()
+        RETURN_NULL();
     }
 }
 
@@ -1385,7 +1426,7 @@ static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event)
     client->suspending = 0;
     client->cid = 0;
     php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-    int ret = coro_resume(sw_current_context, result, &retval);
+    int ret = sw_coro_resume(sw_current_context, result, retval);
     sw_zval_free(result);
 
     if (ret == CORO_END && retval)
@@ -1434,7 +1475,7 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
     client->cid = 0;
 
     php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-    int ret = coro_resume(sw_current_context, result, &retval);
+    int ret = sw_coro_resume(sw_current_context, result, retval);
     zval_ptr_dtor(result);
     if (ret == CORO_END && retval)
     {
@@ -1480,7 +1521,7 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
     client->suspending = 0;
     client->cid = 0;
 
-    int ret = coro_resume(ctx, result, &retval);
+    int ret = sw_coro_resume(ctx, result, retval);
 
     if (ret == CORO_END && retval)
     {
@@ -1799,7 +1840,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             client->cid = 0;
 
             php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-            ret = coro_resume(sw_current_context, result, &retval);
+            ret = sw_coro_resume(sw_current_context, result, retval);
             sw_zval_free(result);
             if (ret == CORO_END && retval)
             {
@@ -1826,7 +1867,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
 
             // always check that is package complete
             // and maybe more responses has already received in buffer, we check it now.
-            if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE && mysql_is_over(client) != SW_OK)
+            if ((client->cmd == SW_MYSQL_COM_QUERY || client->cmd == SW_MYSQL_COM_STMT_EXECUTE) && mysql_is_over(client) != SW_OK)
             {
                 // the **last** sever status flag shows that more results exist but we hasn't received.
                 swTraceLog(SW_TRACE_MYSQL_CLIENT, "need more");
@@ -1851,7 +1892,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             client->cid = 0;
 
             php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-            ret = coro_resume(sw_current_context, result, &retval);
+            ret = sw_coro_resume(sw_current_context, result, retval);
             if (result)
             {
                 sw_zval_free(result);
