@@ -261,7 +261,7 @@ static PHP_METHOD(swoole_process, __construct)
         RETURN_FALSE;
     }
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|bl", &callback, &redirect_stdin_and_stdout, &pipe_type) == FAILURE)
+    if (zend_parse_parameters_throw(ZEND_NUM_ARGS(), "z|bl", &callback, &redirect_stdin_and_stdout, &pipe_type) == FAILURE)
     {
         RETURN_FALSE;
     }
@@ -657,6 +657,52 @@ zend_bool php_swoole_signal_isset_handler(int signo)
     return signal_callback[signo] != NULL;
 }
 
+void php_swoole_process_clean()
+{
+    if (SwooleG.timer.initialized)
+    {
+        swTimer_free(&SwooleG.timer);
+        bzero(&SwooleG.timer, sizeof(SwooleG.timer));
+    }
+
+    swSignal_clear();
+    int i;
+    for (i = 0; i < SW_SIGNO_MAX; i++)
+    {
+        if (signal_callback[i])
+        {
+            sw_zval_free(signal_callback[i]);
+            signal_callback[i] = NULL;
+        }
+    }
+
+    /**
+     * Close EventLoop
+     */
+    if (SwooleG.main_reactor)
+    {
+        SwooleG.main_reactor->free(SwooleG.main_reactor);
+        SwooleG.main_reactor = NULL;
+        swTraceLog(SW_TRACE_PHP, "destroy reactor");
+    }
+
+    SwooleG.memory_pool = swMemoryGlobal_new(SW_GLOBAL_MEMORY_PAGESIZE, 1);
+    if (SwooleG.memory_pool == NULL)
+    {
+        printf("[Process] Fatal Error: global memory allocation failure.");
+        exit(1);
+    }
+
+    bzero(&SwooleWG, sizeof(SwooleWG));
+    SwooleG.pid = getpid();
+
+    if (SwooleG.process_type != SW_PROCESS_USERWORKER)
+    {
+        SwooleG.process_type = 0;
+    }
+
+}
+
 int php_swoole_process_start(swWorker *process, zval *zobject)
 {
     process->pipe = process->pipe_worker;
@@ -686,41 +732,9 @@ int php_swoole_process_start(swWorker *process, zval *zobject)
         }
     }
 
-    /**
-     * Close EventLoop
-     */
-    if (SwooleG.main_reactor)
-    {
-        SwooleG.main_reactor->free(SwooleG.main_reactor);
-        SwooleG.main_reactor = NULL;
-        swTraceLog(SW_TRACE_PHP, "destroy reactor");
-    }
-
-    bzero(&SwooleWG, sizeof(SwooleWG));
-    SwooleG.pid = process->pid;
-    if (SwooleG.process_type != SW_PROCESS_USERWORKER)
-    {
-        SwooleG.process_type = 0;
-    }
+    php_swoole_process_clean();
     SwooleWG.id = process->id;
 
-    if (SwooleG.timer.initialized)
-    {
-        swTimer_free(&SwooleG.timer);
-        bzero(&SwooleG.timer, sizeof(SwooleG.timer));
-    }
-
-    swSignal_clear();
-    int i;
-    for(i=0; i<SW_SIGNO_MAX; i++)
-    {
-        if (signal_callback[i])
-        {
-            sw_zval_free(signal_callback[i]);
-            signal_callback[i] = NULL;
-        }
-    }
-
     zend_update_property_long(swoole_process_class_entry_ptr, zobject, ZEND_STRL("pid"), process->pid);
     zend_update_property_long(swoole_process_class_entry_ptr, zobject, ZEND_STRL("pipe"), process->pipe_worker);
 
