@@ -100,14 +100,11 @@ static void php_swoole_process_pool_onWorkerStart(swProcessPool *pool, int worke
     {
         return;
     }
+
+    php_swoole_process_clean();
     SwooleWG.id = worker_id;
     current_pool = pool;
-    if (SwooleG.main_reactor)
-    {
-        SwooleG.main_reactor->free(SwooleG.main_reactor);
-        SwooleG.main_reactor = NULL;
-        swTraceLog(SW_TRACE_PHP, "destroy reactor");
-    }
+
     if (sw_call_user_function_ex(EG(function_table), NULL, pp->onWorkerStart, &retval, 2, args, 0, NULL) == FAILURE)
     {
         swoole_php_fatal_error(E_WARNING, "onWorkerStart handler error.");
@@ -191,7 +188,7 @@ static void php_swoole_process_pool_onWorkerStop(swProcessPool *pool, int worker
     }
 }
 
-static void php_swoole_process_pool_signal_hanlder(int sig)
+static void php_swoole_process_pool_signal_handler(int sig)
 {
     switch (sig)
     {
@@ -227,7 +224,7 @@ static PHP_METHOD(swoole_process_pool, __construct)
         RETURN_FALSE;
     }
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ll", &worker_num, &ipc_type, &msgq_key) == FAILURE)
+    if (zend_parse_parameters_throw(ZEND_NUM_ARGS(), "l|ll", &worker_num, &ipc_type, &msgq_key) == FAILURE)
     {
         RETURN_FALSE;
     }
@@ -409,9 +406,9 @@ static PHP_METHOD(swoole_process_pool, start)
 
     SwooleG.use_signalfd = 0;
 
-    swSignal_add(SIGTERM, php_swoole_process_pool_signal_hanlder);
-    swSignal_add(SIGUSR1, php_swoole_process_pool_signal_hanlder);
-    swSignal_add(SIGUSR2, php_swoole_process_pool_signal_hanlder);
+    swSignal_add(SIGTERM, php_swoole_process_pool_signal_handler);
+    swSignal_add(SIGUSR1, php_swoole_process_pool_signal_handler);
+    swSignal_add(SIGUSR2, php_swoole_process_pool_signal_handler);
 
     if (pool->ipc_mode > SW_IPC_NONE)
     {
