@@ -368,7 +368,7 @@ static inline int socket_connect(php_stream *stream, Socket *sock, php_stream_xp
     int ret;
     char *ip_address = NULL;
 
-    if (sock->get_fd() < 0)
+    if (unlikely(sock->socket == nullptr))
     {
         return -1;
     }
@@ -818,11 +818,6 @@ static php_stream *socket_create(const char *proto, size_t protolen, const char
     php_stream *stream = NULL;
     Socket *sock;
 
-    if (unlikely(COROG.active == 0))
-    {
-        coro_init();
-    }
-
     php_swoole_check_reactor();
 
     if (strncmp(proto, "unix", protolen) == 0)
@@ -849,6 +844,14 @@ static php_stream *socket_create(const char *proto, size_t protolen, const char
         sock = new Socket(SW_SOCK_TCP);
     }
 
+    if (unlikely(sock->socket == nullptr))
+    {
+        _failed:
+        delete sock;
+        swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
+        return NULL;
+    }
+
     if (FG(default_socket_timeout) > 0)
     {
         sock->set_timeout((double) FG(default_socket_timeout));
@@ -865,8 +868,7 @@ static php_stream *socket_create(const char *proto, size_t protolen, const char
 
     if (stream == NULL)
     {
-        delete sock;
-        return NULL;
+        goto _failed;
     }
     return stream;
 }
@@ -1046,7 +1048,7 @@ static PHP_FUNCTION(_sleep)
     if (num >= 0.001 && sw_coro_is_in())
     {
         php_swoole_check_reactor();
-        RETURN_LONG(swoole_coroutine_sleep((double) num) < 0 ? num : 0);
+        RETURN_LONG(Coroutine::sleep((double ) num) < 0 ? num : 0);
     }
     else
     {
@@ -1071,7 +1073,7 @@ static PHP_FUNCTION(_usleep)
     if (_time >= 0.001 && sw_coro_is_in())
     {
         php_swoole_check_reactor();
-        swoole_coroutine_sleep((double) num / 1000000);
+        Coroutine::sleep((double) num / 1000000);
     }
     else
     {
@@ -1101,7 +1103,7 @@ static PHP_FUNCTION(_time_nanosleep)
     if (_time >= 0.001 && sw_coro_is_in())
     {
         php_swoole_check_reactor();
-        swoole_coroutine_sleep(_time);
+        Coroutine::sleep(_time);
     }
     else
     {
@@ -1160,7 +1162,7 @@ static PHP_FUNCTION(_time_sleep_until)
     if (_time >= 0.001 && sw_coro_is_in())
     {
         php_swoole_check_reactor();
-        swoole_coroutine_sleep(_time);
+        Coroutine::sleep(_time);
     }
     else
     {
