@@ -1,4 +1,5 @@
 <?php
+function main() {
 $cli = new \swoole_http_client("11.11.11.11", 9000);
 
 $cli->on('close', function($cli) {
@@ -9,4 +10,7 @@ $cli->on('error', function($cli) {
     echo "error";
 });
 
-$cli->get('/', function(swoole_http_client $cli) {});
\ No newline at end of file
+$cli->get('/', function(swoole_http_client $cli) {});
+}
+
+main();
