@@ -5,7 +5,8 @@ $ignore_files = [
     $root . "/swoole_async/big_zero.copy",
     $root . "/swoole_client_async/test.jpg",
     $root . "/swoole_process/echo.py",
-    $root . "/swoole_serialize/skipif.inc"
+    $root . "/swoole_serialize/skipif.inc",
+    $root . "/swoole_http_server/httpdata",
 ];
 
 return $ignore_files;
