@@ -4,7 +4,7 @@
 go(function () use ($redis) {
 
     $redis = new co\Redis();
-    $redis->connect('127.0.0.1', 6379);
+    $redis->connect(REDIS_SERVER_HOST, REDIS_SERVER_PORT);
 
 
     go(function () use ($redis) {
