@@ -2,6 +2,11 @@
 __CURRENT__=`pwd`
 __DIR__=$(cd "$(dirname "$0")";pwd)
 
+if [ ! -f "/.dockerenv" ]; then
+    echo "\n❌ This script is just for Docker env!"
+    exit
+fi
+
 #-----------compile------------
 #-------print error only-------
 cd ${__DIR__} && cd ../ && \
@@ -9,11 +14,12 @@ phpize > /dev/null && \
 ./configure \
 --enable-openssl \
 --enable-http2 \
---enable-async-redis \
 --enable-sockets \
 --enable-mysqlnd \
 > /dev/null && \
 make clean > /dev/null && \
-make > /dev/null && make install && \
+make > /dev/null | tee /tmp/compile.log && \
+(test "`cat /tmp/compile.log`" = "" || exit 255) && \
+make install && \
 docker-php-ext-enable swoole && \
 echo "swoole.fast_serialize=On" >> /usr/local/etc/php/conf.d/docker-php-ext-swoole-serialize.ini
